% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ctStanModel.R
\name{ctStanModel}
\alias{ctStanModel}
\title{Convert a frequentist (omx) ctsem model specification to Bayesian (Stan).}
\usage{
ctStanModel(ctmodelobj, type = "stanct", tipredDefault = TRUE)
}
\arguments{
\item{ctmodelobj}{ctsem model object of type 'omx' (default)}

\item{type}{either 'stanct' for continuous time, or 'standt' for discrete time.}

\item{tipredDefault}{Logical. TRUE sets any parameters with unspecified time independent 
predictor effects to have effects estimated, FALSE fixes the effect to zero unless individually specified.}
}
\value{
List object of class ctStanModel, with random effects specified for any intercept type parameters
(T0MEANS, MANIFESTMEANS, and or CINT), and time independent predictor effects for all parameters. Adjust these
after initial specification by directly editing the \code{pars} subobject, so \code{model$pars} .
}
\description{
Convert a frequentist (omx) ctsem model specification to Bayesian (Stan).
}
\examples{
model <- ctModel(type='omx', Tpoints=50,
n.latent=2, n.manifest=1, 
manifestNames='sunspots', 
latentNames=c('ss_level', 'ss_velocity'),
LAMBDA=matrix(c( 1, 'ma1' ), nrow=1, ncol=2),
DRIFT=matrix(c(0, 1,   'a21', 'a22'), nrow=2, ncol=2, byrow=TRUE),
MANIFESTMEANS=matrix(c('m1'), nrow=1, ncol=1),
# MANIFESTVAR=matrix(0, nrow=1, ncol=1),
CINT=matrix(c(0, 0), nrow=2, ncol=1),
DIFFUSION=matrix(c(
  0, 0,
  0, "diffusion"), ncol=2, nrow=2, byrow=TRUE))

stanmodel=ctStanModel(model)


}
